require "defines"
require "upgradestats"




game.oninit(function() OnLoad() end)
game.onload(function() OnLoad() end)
--game.onevent(defines.events.ontick, function() OnTick() end)

game.onevent(defines.events.onbuiltentity, function(event) OnBuilt(event.createdentity) end)
game.onevent(defines.events.onrobotbuiltentity, function(event) OnBuilt(event.createdentity) end)

-- game.onevent(defines.events.onpreplayermineditem, function(event) OnRemoved(event.entity) end)
-- game.onevent(defines.events.onrobotpremined, function(event) OnRemoved(event.entity) end)
-- game.onevent(defines.events.onentitydied, function(event) OnRemoved(event.entity) end)

game.onevent(defines.events.onresearchfinished, function(event) OnResearched(event.research) end)

function OnLoad()
	if not glob.entities then
		glob.entities = {}
		glob.levels = {}
		for _, upgradeName in ipairs(upgradeNames) do
			glob.levels[upgradeName] = 1
		end	
	end
end

function OnTick()
	-- Nope.
end

function OnBuilt(entity)
	-- Is it a gun-turret?
	if string.find(entity.name, escapedEntityBaseName) ~= nil then
		local newName = GetNewName()
		local turret = ReplaceSingleEntity(entity, newName)
		table.insert(glob.entities, turret)
	end
end

function OnRemoved(entity)
	
end

function OnResearched(researchname)
	-- This catches all levels of research. gun-turret-range1, gun-turret-range5, gun-turret-armor3, etc...
	-- Can we find "gun-turret"
	if string.find(researchname, escapedEntityBaseName) ~= nil then
		for _, upgradeName in ipairs(upgradeNames) do
			-- Find the upgrade type. "range" or "armor".
			if string.find(researchname, upgradeName) ~= nil then
				glob.levels[upgradeName] = glob.levels[upgradeName] + 1
				ReplaceEntities()
				break
			end
		end
	end
end

-- This function needs to be rewritten when the base entity is changed.
function ReplaceSingleEntity(turret, newName)
	-- Don't replace a dead or removed turret.
	if not turret.valid then
		return false
	end
	
	-- Get relevant stats of the turret.
	local ammo = GetAmmo(turret)
	local healthPercent = turret.health / game.entityprototypes[turret.name].maxhealth
	
	-- Create a new turret.
	local newTurret = game.createentity{position=turret.position, name=newName, force = turret.force}
	
	-- Set relevant stats on the new turret.
	newTurret.health = healthPercent * game.entityprototypes[newTurret.name].maxhealth
	if ammo then
		newTurret.insert(ammo)
	end
	
	-- Destroy the old turret.
	turret.destroy()
	
	return newTurret
end

function GetNewName()
	-- Build the new name out of the current levels. For example: gun-turret-r3-a5.
	local newName = entityBaseName
	for _, upgradeName in ipairs(upgradeNames) do
		newName = newName .. "-" .. upgradeName .. glob.levels[upgradeName]
	end
	return newName
end

function ReplaceEntities()
	local newName = GetNewName()
	
	newEntities = {}
	-- Replace every turret with a shiny new one.
	for _, entity in pairs(glob.entities) do
		local newEntity = ReplaceSingleEntity(entity, newName)
		if newEntity then
			table.insert(newEntities, newEntity)
		end
	end
	
	glob.entities = newEntities
end


function GetAmmo(turret)
	local inv = turret.getinventory(1)
	
	for name, count in pairs(inv.getcontents()) do
		return {name=name, count=count}
	end
	return false
end
